#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
//填写.h文件的实际路径
#include "list.h"
#include "linkface_sdk_v2.h"

// 固定失败一些入库图片
#define  FAILED_SOME_PIC

extern int linkkit_destroy();

extern int linkkit_init(const char *product_key, const char *device_name, const char *device_secret);

int g_flag_is_running = 1;

void sig_handler(int signo) {
    printf("catch the signal SIGINT %d\n", signo);
    g_flag_is_running = 0;
    exit(0);
}

/* 批量人脸入库回调函数 */
static int _add_user_batch(const struct list_head *pImageInfoList, int num, int *pErrorCode, void *arg) {
    if (!list_empty(pImageInfoList)) {
        LINKFACE_ADD_FACE_IMAGE_LIST_ST *node = NULL;
        LINKFACE_ADD_FACE_IMAGE_LIST_ST *next = NULL;
        int cur_index = 0;
        list_for_each_entry_safe(node, next, pImageInfoList, face_list, LINKFACE_ADD_FACE_IMAGE_LIST_ST) {
#ifdef FAILED_SOME_PIC
            // 固定第二个错误
            if (++cur_index == 1) {
                *(pErrorCode + cur_index) = -1;
                continue;
            }
#endif
            //add_image_info((const char *)(&(node->face_image_info))); //add_image_info由设备厂商实现
            printf("add_image person_id:{%s}, md5:{%s}\n", node->face_image_info.person_id,
                   node->face_image_info.faceimg_md5);
        }
    }
    return 0;
}

/*
 * 用户照片删除
 */
static int _del_image(const linkface_image_info *person, void *arg) {
    printf("del face from local database: %s\n", person->person_id);
    // del_image_info((const char *)person->person_id); //del_image_info由设备厂商实现

    return 0;
}

/*
 * 用户信息添加
 */
static int _add_user(const linkface_user_info *person, void *arg) {
    printf("add user to local database: %s, name: %s\n", person->person_id, person->name);
    //add_user_info((const linkface_user_info*)person); //add_user_info由设备厂商实现

    return 0;
}

/*
 * 用户信息删除
 */
static int _del_user(const linkface_user_info *person, void *arg) {
    printf("del user from local database: %s\n", person->person_id);
    //del_user_info((const char *)person->person_id); //del_user_info由设备厂商实现

    return 0;
}

/*
 * 清空人脸库
 */
static int _clear_all() {
    //clear_data(); //clear_data由设备厂商实现
    return 0;
}

/*
 * 清除指定用户分组
 */
static int _clear_gourp(char *groupid) {
    printf("del group face from local database: %s\n", groupid);
    //clear_gourp_face(groupid);   //clear_gourp_face由设备厂商实现
    return 0;
}

/*
 * 底库人脸照片总数查询回调函数
 */
static int _query_image_base_count(void *arg) {
    int count = 0;

    //count = query_image_count(); //clear_gourp_face由设备厂商实现

    printf("query facebase count : %d\n", count);
    return count;
}

/*
 * 用户信息查询回调函数
 */
static linkface_user_info *_query_user_info(char *person_id, void *arg) {
    linkface_user_info *tmp = NULL;
    int len = sizeof(linkface_user_info);

    tmp = (linkface_user_info *) malloc(len);
    if (tmp == NULL) {
        return NULL;
    }
    memset(tmp, 0, len);

    //query_user_info((const char *)person_id, &tmp); //query_user_info由设备厂商实现
    printf("query user info: %s, name: %s\n", person_id, tmp->name);

    if (tmp->timestamp_ms == 0) {
        free(tmp);
        tmp = NULL;
    }

    return tmp;
}

/*
 * 用户照片查询回调函数
 */
static linkface_image_info *_query_image_info(char *person_id, void *arg) {
    linkface_image_info *tmp = NULL;
    int len = sizeof(linkface_image_info);

    tmp = (linkface_image_info *) malloc(len);
    if (tmp == NULL) {
        return NULL;
    }
    memset(tmp, 0, len);

    //query_image_info((const char *)person_id, &tmp);    //query_image_info由设备厂商实现

    printf("query face info: %s, md5: %s\n", person_id, tmp->faceimg_md5);
    if (strlen(tmp->faceimg_md5) == 0) {
        free(tmp);
        tmp = NULL;
    }
    return tmp;
}

/*
 * 用户照片遍历回调函数
 */
static void
_polling_image_base(long long timestamp, void *arg, void (*cb_sdk)(const linkface_image_info *person, void *user),
                    void *user_sdk) {
    printf("polling_image_base\n");
    //polling_image_info(timestamp, cb_sdk, user_sdk);    //polling_image_info由设备厂商实现
}

/*
 * 用户信息遍历回调函数
 */
static void
_polling_userinfo_base(long long timestamp, void *arg, void (*cb_sdk)(const linkface_user_info *person, void *user),
                       void *user_sdk) {
    printf("polling_userinfo_base\n");
    //polling_user_info(timestamp, cb_sdk, user_sdk);     //polling_user_info由设备厂商实现
}

#define PRODUCT_KEY ""
#define DEVICE_NAME ""
#define DEVICE_SECRET ""

int main(int argc, char **argv) {
    const char *pk = NULL;
    const char *dn = NULL;
    const char *ds = NULL;
    const char *dir = NULL;
    /*
      alarm_interval < 0 直接退出
      alarm_interval = 0 不上报事件
      alarm_interval > 0 代表上报事件间隔
     */
    int alarm_interval = 0;
    if (argc < 5) {
        printf("use default triples, you can modify it by: ./xxx_demo $pk $dn $ds $path \n");
        pk = PRODUCT_KEY;
        dn = DEVICE_NAME;
        ds = DEVICE_SECRET;
        dir = "./";
    } else {
        pk = argv[1];
        dn = argv[2];
        ds = argv[3];
        dir = argv[4];
        alarm_interval = atoi(argv[5]);
    }

    if (signal(SIGINT, sig_handler) == SIG_ERR) {
        perror("signal errror");
        exit(EXIT_FAILURE);
    }

    /* 初始化linkface配置参数。 */
    linkface_config_t cfg;
    memset(&cfg, 0, sizeof(linkface_config_t));

    /* 物联网平台三元组信息. */
    strncpy(cfg.triple.product_key, pk, LINKFACE_PRODUCT_KEY_LEN);
    strncpy(cfg.triple.device_name, dn, LINKFACE_DEVICE_NAME_LEN);
    strncpy(cfg.triple.device_secret, ds, LINKFACE_DEVICE_SECRET_LEN);

    /* 日志输出配置信息. */
    cfg.cfg_log.lvl = LF_LOG_DEBUG;
    cfg.cfg_log.dst = LF_FILE;

    /* 缓存配置信息. */
    cfg.addFaceBatchSize = 10;
    cfg.cfg_cache.logpath = dir;
    cfg.cfg_cache.picpath = dir;
    cfg.cfg_cache.eventpath = dir;
    cfg.cfg_cache.max_size_log_mb = 30;    //日志文件目录100M
    cfg.cfg_cache.max_size_cache_mb = 50;  //缓存图片 500M
    cfg.cfg_cache.max_size_event_mb = 50;

    /* 用户需要实现的接口: 增加底库信息. */
    cfg.config_batch_add_image.cb = _add_user_batch;
    /* 用户需要实现的接口: 删除底库信息. */
    cfg.config_del_image.cb = _del_image;
    /* 用户需要实现的接口: 通过id查询某个底库信息*/
    cfg.config_query_image_info.cb = _query_image_info;
    /* 用户需要实现的接口: 轮询底库信息*/
    cfg.config_polling_image_info.cb = _polling_image_base;
    /* 用户需要实现的接口: 查询底库总数*/
    cfg.config_query_image_count.cb = _query_image_base_count;

    /* 用户需要实现的接口: 增加用户信息 */
    cfg.config_add_user.cb = _add_user;
    /* 用户需要实现的接口: 删除用户信息 */
    cfg.config_del_user.cb = _del_user;
    /* 用户需要实现的接口: 通过id查询用户信息*/
    cfg.config_query_user_info.cb = _query_user_info;
    /* 用户需要实现的接口: 轮询用户信息*/
    cfg.config_polling_user_info.cb = _polling_userinfo_base;

    /* 用户需要实现的接口: 清空底库. */
    cfg.config_clearall.cb = _clear_all;

    cfg.config_clear_usergroup.cb = _clear_gourp;
    /* 确保linkface + linkkit 初始化完成.*/
    while (1) {
        int linkkit_ret = linkkit_init(pk, dn, ds);
        if (linkkit_ret < 0) {
            printf("linkkit sdk init failed, retry 1s later.\n");
            sleep(1);
            if (g_flag_is_running == 0) {
                return 0;
            }
            continue;
        }

        int linkface_ret = linkface_init(&cfg);
        if (linkface_ret < 0) {
            printf("linkface sdk init failed, retry 1s later.\n");
            linkkit_destroy();
            sleep(1);
            if (g_flag_is_running == 0) {
                return 0;
            }
            continue;
        }

        // 保持设备连接状态
//        while (1) {
//            printf("Init done.\n");
//            sleep(10);
//        }

        // 初始化成功就退出
        sleep(10);
        if (linkkit_ret == 1 && linkface_ret == 0) {
            printf("Init done.\n");
            break;
        }
    }

    // 销毁sdk方法
    linkkit_destroy();
    linkface_destroy();
    return 0;
}